/**
 * DisplayProcessor.java
 * Copyright (C) 2012 Pattern Recognition Lab, University Erlangen-Nuremberg.
 *  
 * Licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License (the "License");
 * you may not use this file except in compliance with the License.
 * To view a copy of this License, visit http://creativecommons.org/licenses/by-nc-sa/3.0/.
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on 
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the 
 * specific language governing permissions and limitations under the License.
 * 
 * 
 * This file is part of the "Hearty" Android Application. It was released as supplementary material related to the publication [1]:
 * [1] S. Gradl, P. Kugler, C. Lohmüller, and B. Eskofier, “Real-time ECG monitoring and arrhythmia detection using Android-based mobile devices,” in 34th Annual International Conference of the IEEE EMBS, 2012, pp. 2452–2455.
 * 
 * It is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * If you reuse this code you have to keep or cite this comment.
 *
 */
package de.lme.hearty;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * @author Patrick Kugler
 */
class DisplayProcessor extends ShimmerProcessor {

    private final int BUFFERSIZE = 1024;
    private final int NUM_VALS = 7; // number of consecutive values in the
				    // buffer
    private final int NUM_FRAMES = 10; // number of consecutive frames to store
				       // in buffer before onProcess is called
    private final int PROCESSINGSIZE = NUM_VALS * NUM_FRAMES;

    private int[] ringBuffer;
    private int[] processingBuffer;
    private int counter;
    private int bufferEnd;
    private OutputStream outStream = null;

    public int accel_x;
    public int accel_y;
    public int accel_z;
    public int gyro_x;
    public int gyro_y;
    public int gyro_z;
    public int time_stamp;

    private Lock data_lock;

    public DisplayProcessor(ArrayList<ShimmerDataSource> shimmerDataSources) {
	super(shimmerDataSources);

	this.ringBuffer = new int[BUFFERSIZE];
	this.processingBuffer = new int[PROCESSINGSIZE];
	this.counter = 0;
	this.bufferEnd = 0;

	data_lock = new ReentrantLock();
    }

    public void lock() {
	data_lock.lock();
    }

    public void unlock() {
	data_lock.unlock();
    }

    public void startProcessing() {
	// you can reset some variables here, e.g. clear the buffer
	counter = 0;
	bufferEnd = 0;

	accel_x = 0;
	accel_y = 0;
	accel_z = 0;
	gyro_x = 0;
	gyro_y = 0;
	gyro_z = 0;
	time_stamp = 0;

	// start processing with the found devices
	super.startProcessing();
    }

    @Override
    public void stopProcessing() {
	super.stopProcessing();
    }

    public void onShimmerData(ShimmerDataSource.ShimmerData data) {
	// Called for each new data sample
	// Do not perform expensive computations here!
	// Do not modify the GUI from here!
	try {

	    data_lock.lock();
	    accel_x = data.accel_x;
	    accel_y = data.accel_y;
	    accel_z = data.accel_z;

	    gyro_x = data.sensor_1;
	    gyro_y = data.sensor_2;
	    gyro_z = data.sensor_3;

	    time_stamp = (int) data.timestamp;
	    data_lock.unlock();

	    sendNotification();
	    //
	    // ringBuffer[bufferEnd] = accel_x;
	    // bufferEnd = (bufferEnd + 1) % BUFFERSIZE;
	    // ringBuffer[bufferEnd] = accel_y;
	    // bufferEnd = (bufferEnd + 1) % BUFFERSIZE;
	    // ringBuffer[bufferEnd] = accel_z;
	    // bufferEnd = (bufferEnd + 1) % BUFFERSIZE;
	    // ringBuffer[bufferEnd] = gyro_x;
	    // bufferEnd = (bufferEnd + 1) % BUFFERSIZE;
	    // ringBuffer[bufferEnd] = gyro_y;
	    // bufferEnd = (bufferEnd + 1) % BUFFERSIZE;
	    // ringBuffer[bufferEnd] = gyro_z;
	    // bufferEnd = (bufferEnd + 1) % BUFFERSIZE;
	    // ringBuffer[bufferEnd] = time_stamp;
	    // bufferEnd = (bufferEnd + 1) % BUFFERSIZE;
	    // counter++;
	    //
	    // // check if we have enough data to process
	    // // you can also perform small checks here (check for range,
	    // minimum, etc.)
	    // if (counter == NUM_FRAMES) {
	    // counter = 0;
	    //
	    // // WARNING there shall be dragons!
	    //
	    // // copy the whole array to be processed
	    // if ((bufferEnd-PROCESSINGSIZE) >= 0)
	    // System.arraycopy(ringBuffer, bufferEnd-PROCESSINGSIZE,
	    // processingBuffer, 0, PROCESSINGSIZE);
	    // else
	    // {
	    // System.arraycopy(ringBuffer,
	    // BUFFERSIZE-(PROCESSINGSIZE-bufferEnd), processingBuffer, 0,
	    // PROCESSINGSIZE-bufferEnd );
	    // System.arraycopy(ringBuffer, 0, processingBuffer,
	    // PROCESSINGSIZE-bufferEnd, bufferEnd);
	    // }
	    // // schedule the processing
	    // this.scheduleProcessing();
	    // }
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    protected void onProcess() {
	/*
	 * // Called in a separate thread after scheduleProcessing() was called.
	 * // You can do expensive computations on the processingBuffer here. //
	 * Keep in mind that the ringBuffer may change during execution of this
	 * function! // Do not modify the GUI from here!
	 * 
	 * sum = 0; max = Integer.MIN_VALUE; min = Integer.MAX_VALUE;
	 * 
	 * for (int i=0; i < PROCESSINGSIZE; i++) { sum = sum +
	 * processingBuffer[i]; max = Math.max(max, processingBuffer[i]); min =
	 * Math.min(min, processingBuffer[i]); }
	 */

	// recording functionality changes: output data to recording file
	if (outStream != null) {
	    // Prepare the output string
	    String outstring = "";// processingBuffer.toString();
	    for (int i = 0; i < PROCESSINGSIZE; i++) {
		outstring += processingBuffer[i] + "\n";
	    }
	    // outstring += m_displayProcessor.packet_id + "\t" +
	    // m_displayProcessor.accel_x + "\t" + m_displayProcessor.accel_y +
	    // "\t" + m_displayProcessor.accel_z + "\t" +
	    // m_displayProcessor.gyro_x + "\t" + m_displayProcessor.gyro_y +
	    // "\t" + m_displayProcessor.gyro_z + "\n";
	    // write out the data
	    try {
		outStream.write(outstring.getBytes());
	    } catch (IOException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	    }
	}
	// end recording changes

	// notify the GUI that the variables have changed
	// sendNotification();

    }

    public void setOutStream(OutputStream outStream) {
	this.outStream = outStream;
    }

}
